
-- Fall-Winter Demo 2012
-- turtlesort.com
-- Kevin Sacro

-- A listener for map and player events.
-- Events handled: mapload, enter, leave, and interact.

local eventListener = {}
local lookupTable = {} -- keys:  tileX, tileY, event,
                       -- value: action

function eventListener.notify(event, ...)

	local args = {...}
	
	if event == 'mapload' then
		eventListener._mapLoadEvent(args)
	elseif event == 'enter' or event == 'leave' or event == 'interact' then
		-- args in order: event, tileX, tileY, tileWidth
		eventListener._playerEvent(event, args[1], args[2])
	end

	if game.maps.currentMap and game.maps.currentMap.script and game.maps.currentMap.script.notify then
		game.maps.currentMap.script.notify(event, ...)
	end
	
end

function eventListener._mapLoadEvent(args)

	lookupTable = {} -- Clear our current fields table

	-- Get all fields of current map
	if game.maps.currentMap and game.maps.currentMap.layers.Events then
		
		local fields = game.maps.currentMap.layers.Events.objects
		
		for _,field in pairs(fields) do
		
			-- Field attributes:
			-- visible, type, layer,
			-- x, y, width, height (tile-based)
			-- properties (table of key-value pairs)

			-- Any tile that the field "touches" is part of the field
			local startTileX = math.floor(field.x/game.tilewidth)
			local startTileY = math.floor(field.y/game.tileheight)
			local endTileX = math.floor((field.x + field.width)/game.tilewidth)
			local endTileY = math.floor((field.y + field.height)/game.tileheight)

			-- print(startTileX .. ',' .. startTileY .. ' to ' .. endTileX .. ',' .. endTileY)

			-- Store the trigger/actions in a hashtable where x,y,event are the keys
			for i = startTileX, endTileX, 1 do
				for j = startTileY, endTileY, 1 do
					
					if not lookupTable[i] then lookupTable[i] = {} end
					if not lookupTable[i][j] then lookupTable[i][j] = {} end
										
					for action, actionArgs in pairs(field.properties) do
					
						if not lookupTable[i][j][action] then lookupTable[i][j][action] = {} end
						table.insert(lookupTable[i][j][action], actionArgs)					
						
						-- print('DEBUG:' .. i .. ',' .. j .. ',' .. action .. ' = ' .. actionArgs)
					end
					
				end	
			end -- trigger/actions
		end -- fields
		
	end -- if
end

-- Called on player leave, enter, and interact.
function eventListener._playerEvent(event, tileX, tileY)
		
		if lookupTable[tileX] and
		   lookupTable[tileX][tileY] and
		   lookupTable[tileX][tileY][event] then
			
			for _, commands in pairs(lookupTable[tileX][tileY][event]) do
				local tokens = game.utf8.split2(commands,':')
				local command = game.utf8.trim(tokens[1])

				if command == 'warpto' then
					eventListener._warpToCommand(tokens[2])
					
				elseif command == 'call' then
					eventListener._callCommand(tokens[2])
				end
				
			end
			
		end

end


function eventListener._warpToCommand(args)
	
	args         = game.utf8.split2(args, ",") -- "map_id, tileX, tileY, direction"
	local map_id = game.utf8.trim(args[1])
	local tileX  = game.utf8.trim(args[2])
	local tileY  = game.utf8.trim(args[3])
	
	local fadeOutComplete = function()
		--print("DEBUG: fadeOutComplete on" .. game.maps.currentMap._id)
		if not args[4] then
			game.player.moveTo(tileX, tileY)
		else
			game.player.moveTo(tileX, tileY, game.utf8.trim(args[4])) -- 'direction'
		end

		game.maps.setCurrentMap(map_id)

		if game.maps.currentMap.script and game.maps.currentMap.script.notify then
			game.maps.currentMap.script.notify("mapfadeoutcomplete",map_id)
		end
	
		game.maps.fadeIn(function()
			game.player.canMove = true
			
			if game.maps.currentMap.script and game.maps.currentMap.script.notify then
				game.maps.currentMap.script.notify("mapfadeincomplete",map_id)
			end
			
		end)
		
	end
	
	game.player.canMove = false
	game.maps.fadeOut(fadeOutComplete)
	
	if game.maps.currentMap.script and game.maps.currentMap.script.notify then
		game.maps.currentMap.script.notify("mapfadeoutstart",map_id)
	end

end

function eventListener._callCommand(args)
	
	args               = game.utf8.split2(args, ",") -- "functionName, args, ..."
	local functionName = game.utf8.trim(args[1])
	
	if game.maps.currentMap.script and game.maps.currentMap.script[functionName] then
		
		game.maps.currentMap.script[functionName]()
		
	else
		if not game.maps.currentMap.script then
			print('EventListener: Call to function ' .. functionName .. ' failed because a script does not exist for this map.')
		else
			print('EventListener: Call to function ' .. functionName .. ' failed because the function name is not defined in the map script.')
		end
		
	end
	
end

return eventListener