--[[
	Copyright (c) 2012 Kevin Sacro

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the "Software"),
	to deal in the Software without restriction, including without limitation
	the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the
	Software is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included
	in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
	NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
	DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
	OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
	USE OR OTHER DEALINGS IN THE SOFTWARE.
--]]

-- Handles maps.

-- Map indexes and tileset indexes always start at 1
-- (corresponding to Lua arrays). Map coordinates,
-- map tile coordinates, and tileset tile coordinates
-- always have 0,0 as the origin.

local lib = {}

local tilesets = {}
local maps = {}

local currentTileset
local currentMap

-- Adds a map to the map system with a desired
-- id.
function lib.addMap(id, luamap, script)

	local newmap = {}

	if not luamap then
		luamap = require('maps.' .. id)
	end
		
	if love.filesystem.exists('maps/' .. id .. '-script.lua') then
		newmap.script = require('maps.' .. id .. '-script')
	end
	
	newmap.id = id
	newmap.cols = luamap.width
	newmap.rows = luamap.height

	newmap.tilesetid = luamap.tilesets[1].name
	
	-- If we've never seen this tileset before, process and
	-- cache its quads.
	if not tilesets[newmap.tilesetid] then
		lib._addTileset(luamap)
	end
	
	newmap.width = luamap.width * tilesets[newmap.tilesetid].tilewidth
	newmap.height = luamap.height * tilesets[newmap.tilesetid].tileheight
	
	-- The actual map data
	newmap.static = {}
	newmap.details = {}
	newmap.objects = {}
	newmap.events = {}
	
	for i = 1, #luamap.layers do
	
		local layername = luamap.layers[i].name
	
		if layername == "Static" then
			newmap.static.data = luamap.layers[i].data
			
		elseif layername == "Details" then
			newmap.details.data = luamap.layers[i].data
			
		elseif layername == "Moveable" then
			newmap.objects.initdata = luamap.layers[i].data
			newmap.objects.data = lib._loadObjects(newmap)
			
		elseif layername == "Events" then
			newmap.events.data = luamap.layers[i].objects			
		end
	end
	
	maps[id] = newmap

end

-- Internal function. Extracts the tileset data
-- from the specified map
function lib._addTileset(luamap)
	
	local luatileset = luamap.tilesets[1]
	local tileset = {}
	
	tileset.id = luatileset.name
	tileset.imagename = luatileset.image
	tileset.image = g.newImage('maps/' .. tileset.imagename)
	tileset.tilewidth = luatileset.tilewidth
	tileset.tileheight = luatileset.tileheight
	tileset.width = luatileset.imagewidth
	tileset.height = luatileset.imageheight
	tileset.cols = tileset.width / tileset.tilewidth
	tileset.rows = tileset.height / tileset.tileheight
	tileset.totaltiles = tileset.cols * tileset.rows
	
	tileset.solidlayer = tileset.cols * 5 + 1 -- First five rows
	tileset.abovelayer = tileset.cols * (tileset.rows - 2) + 1 -- Last two rows
	
	-- Build quadcache right away
	tileset.quadcache = {}

	local tilesetindex = 1
	for y = 0, tileset.rows - 1 do
		for x = 0, tileset.cols - 1 do

			local tilemapx = (tilesetindex - 1) % tileset.cols
			local tilemapy = math.floor((tilesetindex - 1)/tileset.cols)
			
			-- And finally the exact coordinates for the tile on tileset
			local pixelx = tilemapx * tileset.tilewidth
			local pixely = tilemapy * tileset.tileheight

			tileset.quadcache[tilesetindex] = love.graphics.newQuad(
				pixelx,
				pixely,
				tileset.tilewidth,
				tileset.tileheight,
				tileset.width,
				tileset.height
			)
			
			tilesetindex = tilesetindex + 1
		end
	end
	
	tilesets[tileset.id] = tileset
	
end


-- Internal function. Converts ObjectLayer elements to more convenient
-- format.
function lib._loadObjects(map)

	local objects = {}

	for i = 1, #map.objects.initdata do
	
		if map.objects.initdata[i] ~= 0 then
		
			local object = {}
			local tileset = tilesets[map.tilesetid]

			object.tilesetindex = map.objects.initdata[i]
			object.tilex = (i - 1) % map.cols
			object.tiley = math.floor((i-1)/map.cols)
			object.x = object.tilex * tileset.tilewidth
			object.y = object.tiley * tileset.tileheight
			object.speed = 150
			object.tick = 0
			-- print((i-1) .. '/' .. map.cols .. ' = ' .. object.tiley)
			
			-- Assign object dts here
			if object.tilesetindex == 'water' then
				object.update = function(dt)
					object.tick = object.tick + dt
				end
			end
			
			table.insert(objects, object)
		
		end
	
	end
	
	return objects
	
end

function lib.setCurrentMap(mapname)
	if maps[mapname] then
		currentTileset = tilesets[maps[mapname].tilesetid]
		currentMap = maps[mapname]
	end
end

function lib.getCurrentMap()
	return currentMap
end

function lib.getCurrentTileset()
	return currentTileset
end

function lib.getMap(mapname)
	return maps[mapname]
end

function lib.getEvent(tilex,tiley)

	local x = tilex * currentTileset.tilewidth + currentTileset.tilewidth/2
	local y = tiley * currentTileset.tileheight + currentTileset.tileheight/2

	for i=1, #currentMap.events.data do
		
		local event = currentMap.events.data[i]
		
		if x > event.x and x < event.x + event.width and y > event.y  and y < event.y + event.height then
			return event.properties
		end
		
	end

end

function lib.isSolid(tilex,tiley)

	local tilesetindex = tilex + currentMap.cols*tiley + 1
	-- first five rows and last two rows of tileset are not solid
	if tilesetindex < 1 or tilesetindex > currentMap.cols*currentMap.rows then
		return true
		
	elseif currentMap.static.data[tilesetindex] >= currentTileset.solidlayer and currentMap.static.data[tilesetindex] < currentTileset.abovelayer then
		return true
		
	elseif currentMap.details.data[tilesetindex] >= currentTileset.solidlayer and currentMap.details.data[tilesetindex] < currentTileset.abovelayer then
		return true
		
	else

		local obj = lib.getObject(tilex,tiley)
		if obj then
			return obj.tilesetindex >= currentTileset.solidlayer and obj.tilesetindex < currentTileset.abovelayer
		else
			return false
		end
		
	end
	
end

function lib.getObject(tilex,tiley)
	
	for i=1, #currentMap.objects.data do
		local obj = currentMap.objects.data[i]
		if obj.tilex == tilex and obj.tiley == tiley then
			return obj
		else
			-- DEBUG: print(tilex .. ',' .. tiley .. ' ?= ' .. obj.tilex .. ',' .. obj.tiley)
		end
	end
	
	return nil
end

-- Updates game entities
function lib.update(dt)

	for i=1, #currentMap.objects.data do
		local obj = currentMap.objects.data[i]
		local targetx = obj.tilex * currentTileset.tilewidth
		local targety = obj.tiley * currentTileset.tileheight
		
		if obj.x ~= targetx or obj.y ~= targety then
			print('1')
				-- Move
			if obj.x < targetx then
				obj.x = obj.x + obj.speed * dt
				if obj.x > targetx then 
					obj.x = targetx
				end
			elseif obj.x > targetx then
				obj.x = obj.x - obj.speed * dt
				if obj.x < targetx then 
					obj.x = targetx
				end
			end
			
			if obj.y < targety then
				obj.y = obj.y + obj.speed * dt
				if obj.y > targety then 
					obj.y = targety
				end
			elseif obj.y > targety then
				obj.y = obj.y - obj.speed * dt
				if obj.y < targety then 
					obj.y = targety
				end
			end
			
		end -- if tilex matches actual x
	end -- for

end

function lib.draw(scenex, sceney, cameratilex, cameratiley)

	--
	for y = 0, currentMap.rows - 1 do
		for x = 0, currentMap.cols - 1 do

			if math.abs(cameratilex - x) <= 9 and math.abs(cameratiley - y) <= 7 then
			
				local tilesetindex = currentMap.static.data[x + currentMap.cols*y + 1]
				
				if tilesetindex > 0 and tilesetindex <= currentTileset.abovelayer then
					g.drawq(
						currentTileset.image,
						currentTileset.quadcache[tilesetindex],
						scenex + x*currentTileset.tilewidth,
						sceney + y*currentTileset.tileheight
					)
				end
				
				tilesetindex = currentMap.details.data[x + currentMap.cols*y + 1]
				
				if tilesetindex > 0 and tilesetindex <= currentTileset.abovelayer then
					g.drawq(
						currentTileset.image,
						currentTileset.quadcache[tilesetindex],
						scenex + x*currentTileset.tilewidth,
						sceney + y*currentTileset.tileheight
					)
				end
			end
			
		end
	end
	--]]
	
	for i = 1, #currentMap.objects.data do
		
		local object = currentMap.objects.data[i]
		
		if object.tilesetindex > 0 and object.tilesetindex <= currentTileset.abovelayer then
			
			if math.abs(cameratilex - object.tilex) <= 9 and math.abs(cameratiley - object.tiley) <= 7 then
				g.drawq(
					currentTileset.image,
					currentTileset.quadcache[object.tilesetindex],
					scenex + object.x,
					sceney + object.y
				)

			end
			
		end
		
	end
end

function lib.drawAbove(scenex, sceney, cameratilex, cameratiley)

	for y = 0, currentMap.rows - 1 do
		for x = 0, currentMap.cols - 1 do
			
			if math.abs(cameratilex - x) <= 9 and math.abs(cameratiley - y) <= 7 then
			
				local tilesetindex = currentMap.details.data[x + currentMap.cols*y + 1]
				
				if tilesetindex >= currentTileset.abovelayer then
					g.drawq(
						currentTileset.image,
						currentTileset.quadcache[tilesetindex],
						scenex + x*currentTileset.tilewidth,
						sceney + y*currentTileset.tileheight
					)
				end
				
			end
			
		end
	end

	for i = 1, #currentMap.objects.data do
		
		local object = currentMap.objects.data[i]
		if object.tilesetindex >= currentTileset.abovelayer then
		
			if math.abs(cameratilex - object.tilex) <= 9 and math.abs(cameratiley - object.tiley) <= 7 then
				g.drawq(
					currentTileset.image,
					currentTileset.quadcache[object.tilesetindex],
					scenex + object.x,
					sceney + object.y
				)
			end
		
		end
		
	end
end

return lib