
-- Fall-Winter Demo 2012
-- turtlesort.com
-- Kevin Sacro

-- The player object responsible for setting up the
-- character sprite animation, drawing the sprite,
-- receiving and handling input from player, and
-- updating player movement and sprite state based
-- on input and previous state.

-- local player = {}
local player = require('lib.observable').new()
local ani    = require 'lib.animation' -- Required for walking animation.

-- Basic data used for collision, movement, sprite centering, etc.
player.x         = game.tilewidth  * 3
player.y         = game.tileheight * 7
player.width     = game.tilewidth  - 11 -- should be smaller than a tile
player.height    = game.tileheight /2   -- should be smaller than a tile
player.speed     = 250

-- Used to notify observers when the player leaves or enters a
-- tile.
player.tileX = -1
player.tileY = -1

-- Any sprite sheet with 4 rows and 4 columns will work by just
-- modifying this next line.
player.image = love.graphics.newImage('opengameart/girl_knight_48x54.png')

-- These two variables determine the graphical state of the player
-- and determine what to draw from the sprite sheet.
player.direction = 'down'
player.isMoving = false
player.canMove = true

-- Sprite frames to use when the player is not walking around.
player.standing = {}
player.standing.up    = love.graphics.newQuad(48*1, 54*0, 48, 54, 48*4, 54*4)
player.standing.right = love.graphics.newQuad(48*1, 54*1, 48, 54, 48*4, 54*4)
player.standing.down  = love.graphics.newQuad(48*1, 54*2, 48, 54, 48*4, 54*4)
player.standing.left  = love.graphics.newQuad(48*1, 54*3, 48, 54, 48*4, 54*4)

-- Sprite frames to use when the player is walking around.
player.moving = {}
player.moving.speed = 0.14 -- This is not walk speed, it is speed of the animation.
player.moving.up    = ani.newAnimation(player.image, 4, 4, player.moving.speed, 1, 4)
player.moving.right = ani.newAnimation(player.image, 4, 4, player.moving.speed, 5, 8)
player.moving.down  = ani.newAnimation(player.image, 4, 4, player.moving.speed, 9, 12)
player.moving.left  = ani.newAnimation(player.image, 4, 4, player.moving.speed, 13, 16)

function player.moveTo(tileX, tileY, direction)
	player.x = tileX * game.tilewidth + game.tilewidth/2 - player.width/2 or 1
	player.y = tileY * game.tileheight + game.tileheight/2 - player.height/2 or 1
	if direction then player.direction = direction end
	
	player.tileX = tileX
	player.tileY = tileY
end


-- Handles collision, player input, notifying observers of events, and
-- graphical state of the sprite (but doesn't draw).
function player.update(dt)
	player.isMoving = false

	if player.canMove then 
		if not player.isMoving and love.keyboard.isDown('left','a') then
		
			player.x = player.x - dt*player.speed -- Move left
			player.direction = 'left'
			player.isMoving = true
			
			 -- Check if left-most corners of player are now in a solid tile
			tilex = player._toTileX(player.x)
			if player._getTile(player.x,player.y) or player._getTile(player.x,player.y + player.height)then
			
				-- If at least one corner is now in a solid tile, move the player to the right just enough
				-- so that no part is inside the solid tile.
				player.x = (tilex + 1) * game.tilewidth
			end
		
		end
		if not player.isMoving and love.keyboard.isDown('right','d') then
		
			player.x = player.x + dt*player.speed
			player.direction = 'right'
			player.isMoving = true
			
			tilex = player._toTileX(player.x + player.width)
			if player._getTile(player.x + player.width, player.y ) or player._getTile(player.x + player.width,player.y + player.height)then
				
				-- Here the -1 is necessary so that right-hand corners don't land inside the solid tile again
				player.x = tilex * game.tilewidth - player.width - 1  
			end
		end	
		if love.keyboard.isDown('up','w') then
		
			player.y = player.y - dt*player.speed
			player.direction = 'up'
			player.isMoving = true
			
			tiley = player._toTileY(player.y)
			if player._getTile(player.x,player.y) or player._getTile(player.x + player.width, player.y)then
				player.y = (tiley + 1) * game.tileheight
			end
		end	
		if love.keyboard.isDown('down','s') then
		
			player.y = player.y + dt*player.speed
			player.direction = 'down'
			player.isMoving = true
			
			tiley = player._toTileY(player.y + player.height)
			if player._getTile(player.x,player.y + player.height) or player._getTile(player.x + player.width, player.y + player.height)then
				player.y = tiley * game.tileheight - player.height - 1 -- -1 so the corner lands in correct tile
			end
			
		end
	end

	player.newTileX = math.floor((player.x + player.width/2)/game.tilewidth)
	player.newTileY = math.floor((player.y + player.height/2)/game.tileheight)
	
	if player.newTileX ~= player.tileX or player.newTileY ~= player.tileY then
		player.notifyObservers('leave', player.tileX, player.tileY, game.maps.currentMap._id)
		player.tileX = player.newTileX
		player.tileY = player.newTileY
		player.notifyObservers('enter', player.tileX, player.tileY, game.maps.currentMap._id)
	end
	
	if player.isMoving then
		player.moving[player.direction]:loop()
		player.moving[player.direction].x = math.floor(player.x) - 6
		player.moving[player.direction].y = math.floor(player.y) - 28 - 2
		player.moving[player.direction]:update(dt)
	else
		player.moving[player.direction]:stop()
	end

	
end

-- Draws the player.
function player.draw()

	if player.drawCollisionBox then
		love.graphics.setColor(255,0,0)
		love.graphics.rectangle("line", math.floor(player.x), math.floor(player.y), player.width, player.height)
	end
	
	love.graphics.setColor(255,255,255)
	
	if not player.isMoving then
		love.graphics.drawq(player.image, player.standing[player.direction], math.floor(player.x) - 6, math.floor(player.y) - 28)
	else
		player.moving[player.direction]:draw()
	end

end

-- Handles interaction events.
function player.keypressed(key,unicode)
	
	if not player.canMove then return end
	
	if key == ' ' or key == 'return' then
		if player.direction == 'up' then
			player.notifyObservers('interact', player.tileX, player.tileY - 1)
			
		elseif player.direction == 'right' then
			player.notifyObservers('interact', player.tileX + 1, player.tileY)
		
		elseif player.direction == 'left' then
			player.notifyObservers('interact', player.tileX - 1, player.tileY)
		
		elseif player.direction == 'down' then
			player.notifyObservers('interact', player.tileX, player.tileY + 1)
		end
	end

end

-- Expecting player's x coordinate with regard to map origin
function player._toTileX(x)
	return math.floor((x)/game.tilewidth)
end

-- Expecting player's y coordinate with regard to map origin
function player._toTileY(y)
	return math.floor((y)/game.tileheight)
end

-- Get's the tile object from the Player layer at the
-- specified x,y coordinate (wrt map origin). Primarily used
-- for collision detection.
function player._getTile(x,y)
	return game.maps.currentMap.layers.Player(player._toTileX(x),player._toTileY(y))
end

return player